@@ -1,4 +1,5 @@
 import argparse
+import base64
 from collections import OrderedDict
 import json
 import logging
@@ -18,21 +19,22 @@ from aiohttp.web import Request
 from aiohttp.web import middleware
 
 from . import _get_version
+from . import trace_snapshot
+from . import tracestats_snapshot
 from .checks import CheckTrace
 from .checks import Checks
 from .checks import start_trace
-from .snapshot import DEFAULT_SNAPSHOT_IGNORES
-from .snapshot import generate_snapshot
-from .snapshot import snapshot
 from .trace import Trace
 from .trace import TraceMap
-from .trace import decode_v04
-from .trace import decode_v05
+from .trace import decode_v04 as trace_decode_v04
+from .trace import decode_v05 as trace_decode_v05
 from .trace import pprint_trace
 from .trace import v04TracePayload
 from .trace_checks import CheckMetaTracerVersionHeader
 from .trace_checks import CheckTraceContentLength
 from .trace_checks import CheckTraceCountHeader
+from .tracestats import decode_v06 as tracestats_decode_v06
+from .tracestats import v06StatsPayload
 
 
 _Handler = Callable[[Request], Awaitable[web.Response]]
@@ -173,14 +175,28 @@ class Agent:
                         tracemap[trace_id].append(span)
         return list(tracemap.values())
 
+    async def _tracestats_by_session(
+        self, token: Optional[str]
+    ) -> List[v06StatsPayload]:
+        stats: List[v06StatsPayload] = []
+        for req in self._requests_by_session(token):
+            if req.match_info.handler == self.handle_v06_tracestats:
+                s = await self._decode_v06_tracestats(req)
+                stats.append(s)
+        return stats
+
     async def _decode_v04_traces(self, request: Request) -> v04TracePayload:
         content_type = request.content_type
         raw_data = await request.read()
-        return decode_v04(content_type, raw_data)
+        return trace_decode_v04(content_type, raw_data)
 
     async def _decode_v05_traces(self, request: Request) -> v04TracePayload:
         raw_data = await request.read()
-        return decode_v05(raw_data)
+        return trace_decode_v05(raw_data)
+
+    async def _decode_v06_tracestats(self, request: Request) -> v06StatsPayload:
+        raw_data = await request.read()
+        return tracestats_decode_v06(raw_data)
 
     async def handle_v04_traces(self, request: Request) -> web.Response:
         return await self._handle_traces(request, version="v0.4")
@@ -188,6 +204,17 @@ class Agent:
     async def handle_v05_traces(self, request: Request) -> web.Response:
         return await self._handle_traces(request, version="v0.5")
 
+    async def handle_v06_tracestats(self, request: Request) -> web.Response:
+        self._requests.append(request)
+        stats = await self._decode_v06_tracestats(request)
+        nstats = len(stats["Stats"])
+        log.info(
+            "received /v0.6/stats payload with %r stats bucket%s",
+            nstats,
+            "s" if nstats else "",
+        )
+        return web.HTTPOk()
+
     async def _handle_traces(
         self, request: Request, version: Literal["v0.4", "v0.5"]
     ) -> web.Response:
@@ -248,6 +275,7 @@ class Agent:
         return web.HTTPOk()
 
     async def handle_snapshot(self, request: Request) -> web.Response:
+        """Generate a snapshot or perform a snapshot test."""
         token = request["session_token"]
         snap_dir = request.url.query.get("dir", request.app["snapshot_dir"])
         snap_ci_mode = request.app["snapshot_ci_mode"]
@@ -273,34 +301,78 @@ class Agent:
             elif "file" in request.url.query:
                 snap_file = request.url.query.get("file")
             else:
-                snap_file = os.path.join(snap_dir, f"{token}.json")
-            frame.add_item(f"File: {snap_file}")
-            log.info("using snapshot file %r", snap_file)
+                snap_file = os.path.join(snap_dir, token)
 
-            snap_path_exists = os.path.exists(snap_file)
-            if snap_ci_mode and not snap_path_exists:
+            # The logic from here is mostly duplicated for traces and trace stats.
+            # If another data type is to be snapshotted then it probably makes sense to abstract away
+            # the required pieces of snapshotting (loading, generating and comparing).
+
+            # For backwards compatibility traces don't have a postfix of `_trace.json`
+            trace_snap_file = f"{snap_file}.json"
+            tracestats_snap_file = f"{snap_file}_tracestats.json"
+
+            frame.add_item(f"Trace File: {trace_snap_file}")
+            frame.add_item(f"Stats File: {tracestats_snap_file}")
+            log.info(
+                "using snapshot files %r and %r", trace_snap_file, tracestats_snap_file
+            )
+
+            trace_snap_path_exists = os.path.exists(trace_snap_file)
+
+            received_traces = await self._traces_by_session(token)
+            if snap_ci_mode and received_traces and not trace_snap_path_exists:
                 raise AssertionError(
-                    f"Snapshot file '{snap_file}' not found. "
+                    f"Trace snapshot file '{trace_snap_file}' not found. "
                     "Perhaps the file was not checked into source control? "
                     "The snapshot file is automatically generated when the test case is run when not in CI mode."
                 )
-            elif snap_path_exists:
+            elif trace_snap_path_exists:
                 # Do the snapshot comparison
-                received_traces = await self._traces_by_session(token)
-                with open(snap_file, mode="r") as f:
+                with open(trace_snap_file, mode="r") as f:
                     raw_snapshot = json.load(f)
-
-                snapshot(
+                trace_snapshot.snapshot(
                     expected_traces=raw_snapshot,
                     received_traces=received_traces,
                     ignored=span_ignores,
                 )
-            else:
+            elif received_traces:
                 # Create a new snapshot for the data received
-                traces = await self._traces_by_session(token)
-                with open(snap_file, mode="w") as f:
-                    f.write(generate_snapshot(traces))
-                log.info("wrote new snapshot to %r", os.path.abspath(snap_file))
+                with open(trace_snap_file, mode="w") as f:
+                    f.write(trace_snapshot.generate_snapshot(received_traces))
+                log.info(
+                    "wrote new trace snapshot to %r", os.path.abspath(trace_snap_file)
+                )
+
+            # Get all stats buckets from the payloads since we don't care about the other fields (hostname, env, etc)
+            # in the payload.
+            received_stats = [
+                bucket
+                for p in (await self._tracestats_by_session(token))
+                for bucket in p["Stats"]
+            ]
+            tracestats_snap_path_exists = os.path.exists(tracestats_snap_file)
+            if snap_ci_mode and received_stats and not tracestats_snap_path_exists:
+                raise AssertionError(
+                    f"Trace stats snapshot file '{tracestats_snap_file}' not found. "
+                    "Perhaps the file was not checked into source control? "
+                    "The snapshot file is automatically generated when the test case is run when not in CI mode."
+                )
+            elif tracestats_snap_path_exists:
+                # Do the snapshot comparison
+                with open(tracestats_snap_file, mode="r") as f:
+                    raw_snapshot = json.load(f)
+                tracestats_snapshot.snapshot(
+                    expected_stats=raw_snapshot,
+                    received_stats=received_stats,
+                )
+            elif received_stats:
+                # Create a new snapshot for the data received
+                with open(tracestats_snap_file, mode="w") as f:
+                    f.write(tracestats_snapshot.generate(received_stats))
+                log.info(
+                    "wrote new tracestats snapshot to %r",
+                    os.path.abspath(tracestats_snap_file),
+                )
         return web.HTTPOk()
 
     async def handle_session_traces(self, request: Request) -> web.Response:
@@ -308,6 +380,31 @@ class Agent:
         traces = await self._traces_by_session(token)
         return web.json_response(traces)
 
+    async def handle_session_tracestats(self, request: Request) -> web.Response:
+        token = request["session_token"]
+        stats = await self._tracestats_by_session(token)
+        return web.json_response(stats)
+
+    async def handle_session_requests(self, request: Request) -> web.Response:
+        token = request["session_token"]
+        resp = []
+        for req in reversed(self._requests_by_session(token)):
+            if req.match_info.handler not in (
+                self.handle_v04_traces,
+                self.handle_v05_traces,
+                self.handle_v06_tracestats,
+            ):
+                continue
+            resp.append(
+                {
+                    "headers": dict(req.headers),
+                    "body": base64.b64encode(await req.read()).decode(),
+                    "url": str(req.url),
+                    "method": req.method,
+                }
+            )
+        return web.json_response(resp)
+
     async def handle_test_traces(self, request: Request) -> web.Response:
         """Return requested traces as JSON.
 
@@ -384,10 +481,13 @@ def make_app(
             web.put("/v0.4/traces", agent.handle_v04_traces),
             web.post("/v0.5/traces", agent.handle_v05_traces),
             web.put("/v0.5/traces", agent.handle_v05_traces),
+            web.put("/v0.6/stats", agent.handle_v06_tracestats),
             web.get("/test/session/start", agent.handle_session_start),
             web.get("/test/session/clear", agent.handle_session_clear),
             web.get("/test/session/snapshot", agent.handle_snapshot),
             web.get("/test/session/traces", agent.handle_session_traces),
+            web.get("/test/session/stats", agent.handle_session_tracestats),
+            web.get("/test/session/requests", agent.handle_session_requests),
             web.get("/test/traces", agent.handle_test_traces),
             # web.get("/test/benchmark", agent.handle_test_traces),
             web.get("/test/trace/analyze", agent.handle_trace_analyze),
@@ -444,7 +544,9 @@ def main(args: Optional[List[str]] = None) -> None:
         type=Set[str],
         default=set(
             _parse_csv(
-                os.environ.get("SNAPSHOT_IGNORED_ATTRS", DEFAULT_SNAPSHOT_IGNORES)
+                os.environ.get(
+                    "SNAPSHOT_IGNORED_ATTRS", trace_snapshot.DEFAULT_SNAPSHOT_IGNORES
+                )
             )
         ),
         help=(
